package aexp.remoteview;

import android.app.Activity;
import android.content.Intent;
import android.content.ComponentName;
import android.content.ServiceConnection;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.os.IBinder;
import android.os.RemoteException;
import android.util.Log;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.TextView;
import android.widget.RemoteViews;

public class RemoteView extends Activity
{
	private static final String LOG_TAG = "REMOTEVIEW";
    private ITimeViewService timeViewService;
    private TimeViewServiceConnection timeViewServiceConnection;
    private Context context;

    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);
        context = (Context)this;
        initService();
        setContentView( R.layout.main );
        Button getViewButton = (Button)findViewById( R.id.gettimeview );
        getViewButton.setOnClickListener(new View.OnClickListener() {
            public void onClick(View view) {
                if( timeViewService != null ) {
                    try {
                        RemoteViews views = timeViewService.getTimeView();
                        Log.d( LOG_TAG, "RemoteViews: "+views );
                        ViewGroup vg = (ViewGroup)findViewById( R.id.remoteview );
                        View inflatedView = views.apply( context, vg );
                        if( vg.getChildCount() > 0 )
                            vg.removeAllViews();
                        vg.addView( inflatedView );
                        vg.requestLayout();
                        dumpViewGroup( 0, vg );
                    } catch( RemoteException ex ) {
                        Log.e( LOG_TAG, ex.getMessage(), ex );
                    }
                } else
                    Log.d( LOG_TAG, "timeViewService==null" );
     		}
        });

    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        releaseService();
    }


	private void initService() {
        if( timeViewServiceConnection == null ) {
            timeViewServiceConnection = new TimeViewServiceConnection();
            Intent i = new Intent();
            i.setClassName( "aexp.remoteview", "aexp.remoteview.TimeViewService" );
            bindService( i, timeViewServiceConnection, Context.BIND_AUTO_CREATE);
            Log.d( LOG_TAG, "bindService()" );
        } 
	}

	private void releaseService() {
	    if( timeViewServiceConnection != null ) {
	        unbindService( timeViewServiceConnection );	  
	        timeViewServiceConnection = null;
		    Log.d( LOG_TAG, "unbindService()" );
        }
	}

    class TimeViewServiceConnection implements ServiceConnection {
        public void onServiceConnected(ComponentName className, 
			IBinder boundService ) {
            timeViewService = ITimeViewService.Stub.asInterface((IBinder)boundService);
            Log.d( LOG_TAG,"onServiceConnected" );
        }

        public void onServiceDisconnected(ComponentName className) {
            timeViewService = null;
            Log.d( LOG_TAG,"onServiceDisconnected" );
        }
    };

    private void dumpViewGroup( int depth, ViewGroup vg ) {
        Log.d( LOG_TAG, "root view, depth:"+depth+"; view: "+vg );
        for( int i = 0 ; i < vg.getChildCount() ; ++i ) {
            View v = vg.getChildAt( i );
            Log.d( LOG_TAG, "depth: "+depth+"; view: "+v );
            if( v instanceof ViewGroup )
                dumpViewGroup( depth+1, (ViewGroup)v );
        }
    }

}
